from time import time
import RPi.GPIO as GPIO
from copreus.baseclasses.arotaryencoder import ARotaryEncoder, Direction


class RotaryEncoder(ARotaryEncoder):
    """Driver for rotary encoder like the KY-040 with software solutions for debouncing and direction detection.

    The driver entry in the yaml file consists of:
      * ADriver entries
        * topics_pub:
          * rotate - mqtt-translations.rotate-left and mqtt-translations.rotate-right
          * button_pressed - mqtt-translations.button_pressed
          * button_state - mqtt-translations.button_state-open and mqtt-translations.button_state-closed
      * RotaryEncoder entries
        * sw: gpio pin for pressing the rotary encoder
        * dt: dt gpio pin
        * clk: clk gpio pin

    Example:
    driver:
        type: rotaryencoder
        pin_sw:  12
        pin_dt:  16
        pin_clk: 20
        topics-pub:
            rotate: /input1/rotate
            button_pressed: /input1/button/pressed
            button_state:   /input1/button/state
        mqtt-translations:
            rotate-left: LEFT
            rotate-right: RIGHT
            button_pressed: PRESSED
            button_state-open: OPEN
            button_state-closed: CLOSED
    """

    _last_clk = -1  # last clk value - used for rotation direction detection
    _last_time = -1 # time stamp of the last rotation step
    _last_direction = 0  # rotation direction of the last rotation step
    _min_time_gap = 0.100  # minimum time gap between direction changes

    def __init__(self, config, mqtt_client=None, logger=None, stdout_log_level=None, no_gui=None,
                 manage_monitoring_agent=True):
        ARotaryEncoder.__init__(self, config, mqtt_client, logger, logger_name=self.__class__.__name__,
                                stdout_log_level=stdout_log_level, no_gui=no_gui,
                                manage_monitoring_agent=manage_monitoring_agent)
        self._last_time = time()

    def _detect_rotation_direction(self):
        """Event handler for gpio pin 'clk'. Detects the rotation direction and filters out 'rotation bounces' -
        rotation direction changes that happen to fast und are thus considered to originiate in bounces. Publishes
        to the topic 'rotate'."""
        akt_clk = GPIO.input(self._clk)
        akt_time = time()
        self._logger.info("Rotary-event detected.")

        rotate = Direction.NONE

        if akt_clk != self._last_clk:
            rotate = Direction.LEFT
            if GPIO.input(self._dt) != akt_clk:
                rotate = Direction.RIGHT

            #filter
            if akt_time - self._last_time < self._min_time_gap and rotate != self._last_direction:
                self._logger.info("Time between direction change to small - skipping this event.")
                rotate = Direction.NONE
            else:
                self._last_direction = rotate
                self._last_time = akt_time

        self._last_clk = akt_clk
        return rotate


def standalone():
    """Calls the static method RotaryEncoder.standalone()."""
    RotaryEncoder.standalone()


if __name__ == "__main__":
    RotaryEncoder.standalone()



import unittest
import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
import pelops.schema.abstractmicroservice
import copreus.drivers


class TestGetDrivers(unittest.TestCase):
    def test_getdrivers_00_basic(self):
        drivers = copreus.drivers.get_drivers()
        self.assertIsNotNone(drivers)
        self.assertGreater(len(drivers), 0)

    def test_getdrivers_01_no_abstract(self):
        drivers = copreus.drivers.get_drivers()
        drivers = drivers.keys()
        self.assertNotIn("ADRIVER", drivers)
        self.assertNotIn("AEPAPER", drivers)
        self.assertNotIn("AEVENTS", drivers)
        self.assertNotIn("APOLLING", drivers)
        self.assertNotIn("AROTARYENCODER", drivers)
        self.assertNotIn("ASPI", drivers)
        self.assertNotIn("AI2C", drivers)

    def test_getdrivers_02_all_drivers(self):
        getdrivers = copreus.drivers.get_drivers()
        getdrivers = getdrivers.keys()
        drivers = ["ADC", "BME_280", "DAC", "DHT", "EPAPERDIRECT", "EPAPERSIMPLE", "INPUT", "POLLINGINPUT",
                   "OUTPUT", "ROTARYENCODER", "ROTARYENCODER2"]
        for driver in drivers:
            self.assertIn(driver, getdrivers)

if __name__ == '__main__':
    unittest.main()



import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from copreus.drivers.rotaryencoder2 import RotaryEncoder2
from tests_manual.tools import resister_to_pubs

filename = '../tests_unit/config_rotaryencoder2.yaml'
resister_to_pubs(filename)
RotaryEncoder2.standalone(('-c', filename))




def get_schema_parts():
    return {
                "properties": {
                    "spi": {
                        "description": "spi configuration",
                        "type": "object",
                        "properties": {
                            "pin_cs": {
                                "description": "cable select gpio pin (if -1 or not pressent than spi-driver "
                                               "internal cs will be used. e.g. SPI_CE0_N/GPIO08 for device 0.)",
                                "type": "integer"
                            },
                            "bus": {
                                "description": "spi bus",
                                "type": "integer"
                            },
                            "device": {
                                "description": "spi device id",
                                "type": "integer"
                            },
                            "maxspeed": {
                                "description": "maximum connection speed in Hz",
                                "type": "integer"
                            }
                        },
                        "required": ["pin_cs", "bus", "device", "maxspeed"],
                        "additionalItems": False
                    },
                },
                "required": ["spi"],
           }



import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from copreus.drivers.dht import DHT
from pelops.myconfigtools import read_config
from pelops.mymqttclient import MyMQTTClient
from pelops.mylogger import create_logger
import threading
import random
import time


def input_handler_temperature(message):
    print("--- temperature: {}".format(message))


def input_handler_humidity(message):
    print("--- humidity: {}".format(message))


filename = '../tests_unit/config_dht.yaml'
config = read_config(filename)
config["driver"]["poll-interval"] = 9999
logger = create_logger(config["logger"], __name__)
mqttclient = MyMQTTClient(config["mqtt"], logger, True)
mqttclient.connect()
mqttclient.subscribe(config["driver"]["topics-pub"]["temperature"], input_handler_temperature)
mqttclient.subscribe(config["driver"]["topics-pub"]["humidity"], input_handler_humidity)

dht = DHT(config)
dht_thread = threading.Thread(target=dht.run)
dht_thread.start()
dht._is_started.wait()

print("started")
while dht._is_started:
    t = 5 + random.random() * 5
    print("waiting for {} second.".format(t))
    time.sleep(t)
    print("... sending poll-now request")
    mqttclient.publish(config["driver"]["topics-sub"]["poll-now"], config["driver"]["mqtt-translations"]["poll-now"])

print("finished")